# The software in this package is distributed under the GNU General
# Public License version 2 (with a special exception described below).
#
# A copy of GNU General Public License (GPL) is included in this distribution,
# in the file COPYING.GPL.
#
# As a special exception, if other files instantiate templates or use macros
# or inline functions from this file, or you compile this file and link it
# with other works to produce a work based on this file, this file
# does not by itself cause the resulting work to be covered
# by the GNU General Public License.
#
# However the source code for this file must still be made available
# in accordance with section (3) of the GNU General Public License.
#
# This exception does not invalidate any other reasons why a work based
# on this file might be covered by the GNU General Public License.

""" file_helper_unittest.py: unit tests for file_helpers

Tests classes and functions in file_helpers

Should be able to run from python2 and python3!

For help see :py:mod:`unittest`

.. codeauthor:: Intra2net
"""

from __future__ import absolute_import

import unittest

from src import file_helpers

from src.call_helpers import call_and_capture
import os


class FileHelperTester(unittest.TestCase):

    def test_cd(self):
        """ tests the cd context manager """

        test_dir = '/'

        start_cwd = os.getcwd()

        with file_helpers.cd(test_dir):
            self.assertEqual(os.getcwd(), test_dir)
        self.assertEqual(os.getcwd(), start_cwd)


    def test_obj_str(self):
        """ test FilesystemFillState.__str__ """

        # create dummy state
        GIGABYTE = 2**30
        state = file_helpers.FilesystemFillState()
        state.name = 'dummy'
        state.size = 10 * GIGABYTE
        state.used = 9 * GIGABYTE
        state.available = 1 * GIGABYTE
        state.capacity = 90
        state.mount_point = '/not/mounted'

        expect = '[Filesystem dummy mounted at /not/mounted: 9.0G/10G used]'

        self.assertEqual(str(state), expect)


    def test_disc_stats_df(self):
        """ tests get_filesystem_fill_states using df """

        stats = file_helpers.get_filesystem_fill_states(
            method=file_helpers.FS_FILL_METHOD_DF)

        # check number
        code, out, err = call_and_capture(file_helpers.DF_CMD)
        self.assertEqual(code, 0)
        self.assertEqual(len(err), 0)
        self.assertEqual(len(out)-1, len(stats))

        for stat in stats:
            # do numbers make sense?
            self.assertGreaterEqual(stat.size, 0)
            self.assertGreaterEqual(stat.used, 0)
            self.assertLessEqual(stat.used, stat.size)
            self.assertGreaterEqual(stat.available, 0)
            self.assertLessEqual(stat.available, stat.size)
            self.assertGreaterEqual(stat.capacity, 0)
            self.assertLessEqual(stat.capacity, 100)

            # are strings non-empty
            self.assertGreater(len(stat.name), 0)
            self.assertGreater(len(stat.mount_point), 0)

            # does match capacity?
            if stat.size > 0:
                capacity = 100. * stat.used / stat.size
                self.assertLess(abs(capacity - stat.capacity), 5.,
                                'capacity deviates from used/size by >5%!')

            # is size approx equal to used + available?
            size = stat.used + stat.available
            self.assertLess(float(abs(stat.size - size)),
                            0.1 * float(max(stat.size, size)),
                            'size deviates from used+free by more than 10%!')


    def test_compare_methods(self):
        """ compares methods to get_filesystem_fill_states

        turns out STATVFS method finds more filesystem thatn df lists; accept
        that.
        """

        statesDF = file_helpers.get_filesystem_fill_states(
            method=file_helpers.FS_FILL_METHOD_DF)
        statesNF = file_helpers.get_filesystem_fill_states(
            method=file_helpers.FS_FILL_METHOD_STATVFS)

        self.assertLessEqual(len(statesDF), len(statesNF))

        for stateDF in statesDF:
            stateNF = None
            for state in statesNF:
                if state.mount_point == stateDF.mount_point:
                    stateNF = state
                    break
            self.assertNotEqual(stateNF, None)

            # now compare the two
            self.assertEqual(stateDF.size, stateNF.size)
            self.assertLess(abs(stateDF.used - stateNF.used),
                            0.1 * float(stateDF.size))
            self.assertLess(abs(stateDF.available - stateNF.available),
                            0.1 * float(stateDF.size))
            self.assertLess(abs(stateDF.capacity - stateNF.capacity), 5)
            self.assertEqual(stateDF.name, stateNF.name)


    def test_disc_state_statvfs(self):
        for state in file_helpers.get_all_statvfs_fills():
            self.assertEqual(state.used + state.available, state.size)
            self.assertGreaterEqual(state.size, 0)
            self.assertGreaterEqual(state.used, 0)
            self.assertGreaterEqual(state.available, 0)
            self.assertLessEqual(state.used, state.size)
            self.assertLessEqual(state.available, state.size)

            # does match capacity? (would be very strange since capacity is
            # calculated from used and size)
            if state.size == 0:
                self.assertEqual(state.used, 0)
                self.assertEqual(state.available, 0)
            else:
                capacity = 100. * state.used / state.size
                self.assertLess(abs(capacity - state.capacity), 5.,
                                'capacity deviates from used/size by >5%!')

            # is size approx equal to used + available?
            size = state.used + state.available
            self.assertLessEqual(float(abs(state.size - size)),
                            0.1 * float(max(state.size, size)),
                            'size deviates from used+free by more than 10%!')


if __name__ == '__main__':
    unittest.main()
